#!/bin/bash
echo "Starting registry housekeeping ..."

if [ -z $SUPPORTED_VERSIONS ]; then
  echo "[ERROR] Missing environment variable SUPPORTED_VERSIONS" > /dev/stderr
  exit 1
elif [ -z $MAINTAINED_SERVICES ]; then
  echo "[ERROR] Missing environment variable MAINTAINED_SERVICES" > /dev/stderr
  exit 1
elif [ -z $CI_REGISTRY2_URL ]; then
  echo "[ERROR] Missing environment variable CI_REGISTRY2_URL" > /dev/stderr
  exit 1
fi

VERSIONS=(${SUPPORTED_VERSIONS//,/ })
SERVICES=(${MAINTAINED_SERVICES//,/ })

for SERVICE in "${SERVICES[@]}"; do
  TAGS=$(regctl tag ls "${CI_REGISTRY2_URL}/${SERVICE}")
  TAGS=(${TAGS//\n/ })
  for TAG in "${TAGS[@]}"; do
    if [[ ! "${VERSIONS[*]}" =~ $TAG ]]; then
      regctl tag rm ${CI_REGISTRY2_URL}/${SERVICE}:$TAG
      echo "[INFO] Deleted unsupported tag ${CI_REGISTRY2_URL}/${SERVICE}:$TAG"
    fi
  done
done
echo "[INFO] Finished successfully."
