package at.ac.tuwien.ifs.dbrepo.core.api.auth;

import com.fasterxml.jackson.annotation.JsonProperty;
import io.swagger.v3.oas.annotations.media.Schema;
import jakarta.validation.constraints.NotBlank;
import jakarta.validation.constraints.NotNull;
import lombok.*;
import lombok.extern.jackson.Jacksonized;

import java.util.UUID;

@Getter
@Setter
@Builder
@EqualsAndHashCode
@NoArgsConstructor
@AllArgsConstructor
@Jacksonized
@ToString
public class CreateUserDto {

    @NotNull
    @Schema(example = "3b91bc36-3eae-4662-a4be-8993624ab0cb", description = "The user id generated by Keycloak")
    private UUID id;

    @NotNull
    @JsonProperty("ldap_id")
    @Schema(example = "ea022d6d-b4a4-42f3-836f-ff4e596a527a", description = "The user id generated by OpenLDAP")
    private UUID ldapId;

    @NotBlank
    @Schema(example = "user")
    private String username;

    @JsonProperty("given_name")
    @Schema(example = "foo")
    private String givenName;

    @JsonProperty("family_name")
    @Schema(example = "bar")
    private String familyName;

}
